@@ -45,6 +45,14 @@ p3d_same(const POINT3D *p1, const POINT3D *p2)
 	    && FP_EQUALS(p1->z, p2->z);
 }
 
+int
+p3dz_same(const POINT3DZ *p1, const POINT3DZ *p2)
+{
+	return FP_EQUALS(p1->x, p2->x)
+	    && FP_EQUALS(p1->y, p2->y)
+	    && FP_EQUALS(p1->z, p2->z);
+}
+
 int
 p2d_same(const POINT2D *p1, const POINT2D *p2)
 {
@@ -82,7 +90,9 @@ lw_seg_length(const POINT2D *A1, const POINT2D *A2)
 int
 lw_pt_in_arc(const POINT2D *P, const POINT2D *A1, const POINT2D *A2, const POINT2D *A3)
 {
-	return lw_segment_side(A1, A3, A2) == lw_segment_side(A1, A3, P);
+	int pt_side = lw_segment_side(A1, A3, P);
+	int arc_side = lw_segment_side(A1, A3, A2);
+	return pt_side == 0 || pt_side == arc_side;
 }
 
 /**
@@ -96,6 +106,14 @@ lw_pt_in_seg(const POINT2D *P, const POINT2D *A1, const POINT2D *A2)
 	       ((A1->y <= P->y && P->y < A2->y) || (A1->y >= P->y && P->y > A2->y));
 }
 
+int
+lw_pt_on_segment(const POINT2D* A1, const POINT2D* A2, const POINT2D* P)
+{
+	int side = lw_segment_side(A1, A2, P);
+	if (side != 0) return LW_FALSE;
+	return lw_pt_in_seg(P, A1, A2);
+}
+
 /**
 * Returns true if arc A is actually a point (all vertices are the same) .
 */
@@ -405,8 +423,8 @@ int lw_segment_intersects(const POINT2D *p1, const POINT2D *p2, const POINT2D *q
 	** location of the other end-point. Only touches by the first point
 	** will be considered "real" to avoid double counting.
 	*/
-	LWDEBUGF(4, "pq1=%.15g pq2=%.15g", pq1, pq2);
-	LWDEBUGF(4, "qp1=%.15g qp2=%.15g", qp1, qp2);
+	LWDEBUGF(4, "pq1=%d pq2=%d", pq1, pq2);
+	LWDEBUGF(4, "qp1=%d qp2=%d", qp1, qp2);
 
 	/* Second point of p or q touches, it's not a crossing. */
 	if ( pq2 == 0 || qp2 == 0 )
@@ -510,7 +528,7 @@ int lwline_crossing_direction(const LWLINE *l1, const LWLINE *l2)
 
 			this_cross = lw_segment_intersects(p1, p2, q1, q2);
 
-			LWDEBUGF(4, "i=%d, j=%d (%.8g %.8g, %.8g %.8g)", this_cross, i, j, p1->x, p1->y, p2->x, p2->y);
+			LWDEBUGF(4, "i=%d, j=%d (%.8g %.8g, %.8g %.8g)", i, j, p1->x, p1->y, p2->x, p2->y);
 
 			if ( this_cross == SEG_CROSS_LEFT )
 			{
